local const = require("const")
local json = require("json")
local utils = require("utils3")
local shadow = require("shadow2")

utils.check_request_method("POST")
utils.check_auth()
utils.check_permission("DeviceManage")

local function current_time()
    ngx.update_time()
    return ngx.now() * 1000
end

local function call_services(product, device, services)
    local thing_shadow = shadow:new(nil, product, device)

    local resp = {}
    resp["Data"] = {}
    resp["Data"]["Services"] = {}
    for _, s in pairs(services) do
        local name = s["Name"]
        if type(name) ~= "string" or #name == 0 then
            return utils.error_exit(
                    ngx.HTTP_BAD_REQUEST,
                    const.INVALID_PARAMETER,
                    "Service.Name not a string or is empty")
        end
        local args = s["Args"]

        local r = {}
        r["Name"] = name
        r["Returns"] = {}
        local ok, ret, cause = thing_shadow:call_service(name, args)
        if ok then
            r["Returns"]["Data"] = ret
        else
            r["Returns"]["Code"] = ret
            r["Returns"]["Message"] = cause
        end
        table.insert(resp["Data"]["Services"], r)
    end
    resp["Data"]["Timestamp"] = current_time()

    local str = json.encode(resp)
    ngx.header['Content-Type'] = 'application/json; charset=utf-8'
    ngx.say(str)
end

local product = ngx.var[1]
if not utils.is_valid_pk(product) then
    return utils.error_exit(
            ngx.HTTP_BAD_REQUEST,
            const.INVALID_PARAMETER,
            "illegal ProductKey")
end

local device = ngx.var[2]
if not utils.is_valid_dn(device) then
    return utils.error_exit(
            ngx.HTTP_BAD_REQUEST,
            const.INVALID_PARAMETER,
            "illegal DeviceName")
end

ngx.req.read_body()
local body = ngx.req.get_body_data()
if not body then
    return utils.error_exit(
            ngx.HTTP_BAD_REQUEST,
            const.INVALID_PARAMETER,
            "no body found")
end

local ok, args = pcall(json.decode, body)
if not ok then
    return utils.error_exit(
            ngx.HTTP_BAD_REQUEST,
            const.INVALID_PARAMETER,
            "not a JSON body")
end

local services = args["Services"]
if not utils.is_array_table(services) or #services == 0 then
    return utils.error_exit(
            ngx.HTTP_BAD_REQUEST,
            const.INVALID_PARAMETER,
            "Services not a array or is empty")
end

call_services(product, device, services)