local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")
local const = require('const')
local request_method = ngx.var.request_method
local ldbus = require "ldbus"

utils.check_login()
utils.check_permission("DeviceManage")

local function generate_random_num()
    math.randomseed(tostring(os.time()):reverse():sub(1, 7))
    return math.random(1,100)
end

local function get_pk_dn_from_interface(interface, prefix)
    local s = utils.split_str(interface, "_")
    local pk = string.sub(s[1], string.len(prefix)+1, string.len(s[1]))
    local dn = s[2]

    return pk, dn
end

function call_message_route_method(conn, rule, method)
    local msg = assert ( ldbus.message.new_method_call ( const.subscribe.wkn,  const.subscribe.obj, const.subscribe.wkn , method) , "Message Null" )
    local iter = ldbus.message.iter.new ( )
    msg:iter_init_append ( iter )
    assert ( iter:append_basic ( rule ) , "Out of Memory" )
    local reply = assert ( conn:send_with_reply_and_block ( msg ) )
 
    return reply
end


function receive (pk, dn, event_name)
    local prefix = "iot.device.id"
    local postfix = "_"
    local cur_worker_pid = ngx.worker.pid()
    local wkn = "linkedge.webconsole.a"..cur_worker_pid..generate_random_num()
	local device_local_id = prefix..pk..postfix..dn
    
    utils.log_error ( "Listening for signals, pid: " ..cur_worker_pid.." -->:  "..wkn)

	local conn = assert ( ldbus.connection.open ( const.linkedge.bus_addr ) )

    if (ldbus.bus.register(conn, "register failed"))
    then
        utils.log_error("register succeed.")
    else
        utils.log_error("failed to register dbus")
    end
    assert ( ldbus.bus.request_name ( conn , wkn , { replace_existing = true } ) )
    local msg = assert ( ldbus.message.new_method_call ( const.subscribe.wkn,  const.subscribe.obj, const.subscribe.wkn , const.subscribe.interface_sub) , "Message Null" )
    local iter = ldbus.message.iter.new ( )
    msg:iter_init_append ( iter )
    local rule = string.format(const.subscribe.fmt_sub, pk, dn, wkn)
    assert ( iter:append_basic ( rule ) , "Out of Memory" )
    local reply = assert ( conn:send_with_reply_and_block ( msg ) )
    
    assert ( ldbus.bus.add_match ( conn , "type='signal',interface='"..device_local_id.."'" ) )
	conn:flush ( )

	utils.log_error ( "Match rule sent for "..device_local_id)

	while conn:read_write ( 1*1000 ) do
        while true do
		local msg = conn:pop_message ( )
        if msg == nil then
            break
        end
        local res = '{"code": 204, "message": "heartbeats", "data": null}'
        if msg and msg:get_type ( ) == "signal" and msg:get_interface( ) ~= "org.freedesktop.DBus" then
			local iter = ldbus.message.iter.new ( )
			assert ( msg:iter_init ( iter ) , "Message has no parameters" )
			assert ( iter:get_arg_type ( ) == ldbus.types.string , "Argument is not a string" )
			local sigvalue = iter:get_basic ()
            local msg_type = msg:get_member()
            local pk, dn = get_pk_dn_from_interface(msg:get_interface(), prefix)
            local ret_table = {}
            local ret_table_data = {}
            local ret_table_data_items = {}
            ret_table_data_items["productKey"] = pk
            ret_table_data_items["deviceName"] = dn
            ret_table_data_items["eventCode"] = ""
            ret_table_data_items["iotId"] = ""
            ret_table_data_items["batchId"] = ""
            ret_table_data_items["timestamp"] = ""
            ret_table_data_items["eventName"] = msg_type
            ret_table_data_items["eventType"] = msg_type 
            ret_table_data_items["eventBody"] = json.decode(sigvalue) 
            ret_table_data["timestamp"] = ""
            ret_table_data["items"] = ret_table_data_items
            ret_table["code"] = 200
            ret_table["message"] = "success"
            ret_table["data"] = ret_table_data
            
            res = json.encode(ret_table)
        end
        
        ok, err = ngx.say(res)
        ngx.flush(true)

        if ok
            then
                utils.log_error("send status: " ..ok)
            else
                utils.log_error("remote client offline now")
                call_message_route_method(conn, rule, const.subscribe.interface_unsub)
                conn:close()
                ngx.exit(ngx.HTTP_OK)
        end
        end
	end
    utils.log_error("clear dbus connection...")
end

if ("POST" == request_method) then
    local utils_header = require("utils_header")
    local params = utils_header.check_public_body_data()
    local pk = params["productKey"]
    local dn = params["deviceName"]
    local event_name = params["eventIdentifier"]
   
    utils.check_pk(pk)
    utils.check_dn(dn)

    receive (pk, dn, event_name)
end
