local json = require("json")
local utils = require("utils")

utils.check_login()
utils.check_permission("DriverManage")

local request_method = ngx.var.request_method
if request_method ~= "POST" then
    return utils.error_exit(405, "unsupported request method: " .. request_method)
end

local utils_header = require("utils_header")
local params = utils_header.check_public_body_data()
local driver_id = params["DriverId"]
if type(driver_id) ~= "string" or #driver_id == 0 then
    return utils.error_exit(400, "driver id is not string or is empty")
end
local mode, err = utils.get_driver_mode(driver_id)
if mode == nil then
    return utils.error_exit(500, err)
end

local resp = {}
resp["code"] = 200
resp["message"] = "success"
resp["data"] = { Mode = mode }
ngx.say(json.encode(resp))