local utils = require("utils")

utils.check_login()
utils.check_permission("DriverManage")

local request_method = ngx.var.request_method
if request_method ~= "POST" then
    return utils.error_exit(405, "unsupported request method: " .. request_method)
end

local utils_header = require("utils_header")
local params = utils_header.check_public_body_data()
local driver_id = params["DriverId"]
local values = params["Values"]
if type(driver_id) ~= "string" or #driver_id == 0 then
    return utils.error_exit(400, "driver id is not string or is empty")
end
if type(values) ~= "table" then
    return utils.error_exit(400, "driver values is not JSON object")
end

local key = next(values)
if not key then
    return utils.error_exit(400, "driver values is empty")
end

local ok, err = utils.set_driver_values(driver_id, values)
if not ok then
    return utils.error_exit(500, err)
end
utils.error_exit(200)