--- Start to upgrade devices.

local const = require("const")
local json = require("json")
local utils = require("utils")
local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("OTAManage")

if request_method ~= "POST" then
    return utils.error_exit(405, "unsupported request method: " .. request_method)
end

local utils_header = require("utils_header")
local params = utils_header.check_public_body_data()
local driver_id = params["DriverId"]
if type(driver_id) ~= "string" or #driver_id == 0 then
    return utils.error_exit(400, "DriverId is not string or is empty")
end
local firmware_id = params["FirmwareId"]
if type(firmware_id) ~= "string" or #firmware_id == 0 then
    return utils.error_exit(400, "FirmwareId is not string or is empty")
end
local devices = params["Devices"]
if type(devices) ~= "table" then
    return utils.error_exit(400, "Devices not a array")
end

local function start_upgrade(driver_id, devices, firmware_id)
    local firmware = ngx.var.firmware_root .. "/" .. firmware_id
    local updated_devices = {}
    for _, v in ipairs(devices) do
        local d = {}
        d["productKey"] = v["ProductKey"]
        d["deviceName"] = v["DeviceName"]
        table.insert(updated_devices, d)
    end

    local params = {}
    params["driverId"] = driver_id
    params["devices"] = updated_devices
    params["file"] = firmware
    local ok, ret, err = utils.dbus_call_edge(
            const.device_ota.wkn,
            const.device_ota.obj,
            const.device_ota.inf,
            const.device_ota.method.start_upgrade,
            json.encode(params))
    if not ok then
        return false, err
    end
    local ok, res = pcall(json.decode, ret[1])
    if not ok or type(res) ~= "table" then
        return false, "response not a JSON object"
    end
    if res["code"] ~= 0 then
        return false, tostring(res["message"])
    end
    return true
end

local ok, err = start_upgrade(driver_id, devices, firmware_id)
if not ok then
    return utils.error_exit(500, err)
end

local resp = { code = 200, message = "success" }
ngx.say(json.encode(resp))
